@@ -19,8 +19,14 @@ import signal
 import sys
 import time
 import traceback
+import warnings
 from ftplib import error_perm, error_proto, error_reply, error_temp
 
+# silence this warning
+from cryptography.utils import CryptographyDeprecationWarning
+
+warnings.simplefilter("ignore", CryptographyDeprecationWarning)
+
 # 3rd party
 from paramiko.ssh_exception import SSHException
 from scp import SCPClient
@@ -54,13 +60,12 @@ class SplitCopyPut:
         self.copy_proto = kwargs.get("copy_proto")
         self.noverify = kwargs.get("noverify")
         self.use_curses = kwargs.get("use_curses")
+        self.overwrite = kwargs.get("overwrite")
         self.scs = SplitCopyShared(**kwargs)
         self.mute = False
-        self.hard_close = False
         self.sshshell = None
-        self.remote_dir = ""
-        self.remote_file = ""
         self.progress = Progress()
+        self.use_shell = False
 
     def handlesigint(self, sigint, stack):
         """called when SigInt is received
@@ -72,7 +77,7 @@ class SplitCopyPut:
         logger.debug(f"signal {sigint} received, stack:\n{stack}")
         self.mute = True
         self.progress.stop_progress()
-        self.scs.close(hard_close=self.hard_close)
+        self.scs.close()
 
     def put(self):
         """copies file from local host to remote host
@@ -93,43 +98,47 @@ class SplitCopyPut:
         # handle sigint gracefully on *nix
         signal.signal(signal.SIGINT, self.handlesigint)
 
-        # connect to host
+        # determine local file size
+        file_size = self.determine_local_filesize()
+
+        # confirm local storage is sufficient
+        self.scs.storage_check_local(file_size)
+
+        # connect to host, open ssh transport
         self.sshshell, ssh_kwargs = self.scs.connect(SSHShell, **ssh_kwargs)
 
-        # determine remote host os
+        # is this a juniper cli?
+        if self.scs.juniper_cli_check():
+            self.use_shell = True
+            self.scs.enter_shell()
+
+        # ensure dest path is valid
+        remote_file, remote_dir = self.validate_remote_path_put()
+
+        # if target file exists, delete it?
+        self.delete_target_remote(remote_dir, remote_file)
+
+        # determine the OS
         junos, evo, bsd_version, sshd_version = self.scs.which_os()
 
         # verify which protocol to use
         self.copy_proto, self.passwd = self.scs.which_proto(self.copy_proto)
 
-        # ensure dest path is valid
-        self.validate_remote_path_put()
-
-        # delete target file if it already exists
-        if self.check_target_exists():
-            self.delete_target_remote()
-
         # check required binaries exist on remote host
         self.scs.req_binaries(junos=junos, evo=evo)
 
         # cleanup previous remote tmp directory if found
         self.scs.remote_cleanup(
-            remote_dir=self.remote_dir, remote_file=self.remote_file, silent=True
+            remote_dir=remote_dir, remote_file=remote_file, silent=True
         )
 
-        # determine local file size
-        file_size = self.determine_local_filesize()
-
         # determine optimal size for chunks
         split_size, executor = self.scs.file_split_size(
             file_size, sshd_version, bsd_version, evo, self.copy_proto
         )
 
         # confirm remote storage is sufficient
-        self.scs.storage_check_remote(file_size, split_size)
-
-        # confirm local storage is sufficient
-        self.scs.storage_check_local(file_size)
+        self.scs.storage_check_remote(file_size, split_size, remote_dir)
 
         if not self.noverify:
             # get/create sha for local file
@@ -144,7 +153,7 @@ class SplitCopyPut:
             self.progress.add_chunks(file_size, chunks)
 
             # create tmp directory
-            remote_tmpdir = self.scs.mkdir_remote()
+            remote_tmpdir = self.scs.mkdir_remote(remote_dir, remote_file)
 
             # begin connection/rate limit check and transfer process
             command_list = []
@@ -153,7 +162,7 @@ class SplitCopyPut:
             print("starting transfer...")
             self.progress.start_progress(self.use_curses)
             # copy files to remote host
-            self.hard_close = True
+            self.scs.hard_close = True
             loop_start = datetime.datetime.now()
             loop = asyncio.new_event_loop()
             tasks = []
@@ -177,11 +186,10 @@ class SplitCopyPut:
                 self.progress.stop_progress()
                 self.scs.close(
                     err_str="\nan error occurred while copying the files to the remote host",
-                    hard_close=self.hard_close,
                 )
             finally:
                 loop.close()
-            self.hard_close = False
+            self.scs.hard_close = False
             while self.progress.totals["percent_done"] != 100:
                 time.sleep(0.1)
             self.progress.stop_progress()
@@ -190,7 +198,9 @@ class SplitCopyPut:
         loop_end = datetime.datetime.now()
 
         # combine chunks
-        self.join_files_remote(SCPClient, chunks, remote_tmpdir)
+        self.join_files_remote(
+            SCPClient, chunks, remote_dir, remote_file, remote_tmpdir
+        )
 
         # remove remote tmp dir
         self.scs.remote_cleanup()
@@ -200,16 +210,16 @@ class SplitCopyPut:
             self.scs.limits_rollback()
 
         # check remote file size is correct
-        self.compare_file_sizes(file_size)
+        self.compare_file_sizes(file_size, remote_dir, remote_file)
 
         if self.noverify:
             print(
                 f"file has been successfully copied to {self.host}:"
-                f"{self.remote_dir}/{self.remote_file}"
+                f"{remote_dir}/{remote_file}"
             )
         else:
             # generate a sha hash for the combined file, compare to hash of src
-            self.remote_sha_put(sha_bin, sha_len, sha_hash)
+            self.remote_sha_put(sha_bin, sha_len, sha_hash, remote_dir, remote_file)
 
         self.sshshell.close()
         return loop_start, loop_end
@@ -225,9 +235,7 @@ class SplitCopyPut:
             if fnmatch.fnmatch(chunk, f"{self.local_file}*"):
                 chunks.append([chunk, os.stat(chunk).st_size])
         if not chunks:
-            self.scs.close(
-                err_str="file split operation failed", hard_close=self.hard_close
-            )
+            self.scs.close(err_str="file split operation failed")
         # sort alphabetically
         chunks = sorted(chunks)
         logger.info(f"# of chunks = {len(chunks)}")
@@ -239,94 +247,122 @@ class SplitCopyPut:
         :return: None
         """
         logger.info("entering validate_remote_path_put()")
+        remote_path = self.remote_path
         try:
-            self.expand_remote_path()
-            self.path_startswith_tilda()
+            remote_path = self.expand_remote_path(remote_path)
+            remote_path = self.path_startswith_tilda(remote_path)
         except ValueError as err:
             self.scs.close(
                 err_str=err,
-                hard_close=self.hard_close,
             )
 
-        if self.sshshell.run(f"test -d {self.remote_path}")[0]:
+        if self.scs.ssh_cmd(f"test -d {remote_path}")[0]:
             # target path provided is a directory
-            self.remote_file = self.local_file
-            self.remote_dir = self.remote_path.rstrip("/")
-        elif self.sshshell.run(f"test -d {os.path.dirname(self.remote_path)}")[0]:
-            if os.path.basename(self.remote_path) != self.local_file:
+            remote_file = self.local_file
+            remote_dir = remote_path.rstrip("/")
+        elif self.scs.ssh_cmd(f"test -d {os.path.dirname(remote_path)}")[0]:
+            if os.path.basename(remote_path) != self.local_file:
                 # target path provided was a full path, file name does not match src
                 # honour the change of file name
-                self.remote_file = os.path.basename(self.remote_path)
+                remote_file = os.path.basename(remote_path)
             else:
                 # target path provided was a full path, file name matches src
-                self.remote_file = self.local_file
-            self.remote_dir = os.path.dirname(self.remote_path)
+                remote_file = self.local_file
+            remote_dir = os.path.dirname(remote_path)
         else:
             self.scs.close(
-                err_str=f"target path {self.remote_path} on remote host isn't valid",
-                hard_close=self.hard_close,
+                err_str=f"target path {remote_path} on remote host isn't valid",
             )
-        logger.debug(
-            f"remote_dir now = {self.remote_dir}, "
-            f"remote_file now = {self.remote_file}, "
-        )
+        logger.debug(f"remote_dir now = {remote_dir}, remote_file now = {remote_file}")
         # update SplitCopyShared with these values
-        self.scs.remote_dir = self.remote_dir
-        self.scs.remote_file = self.remote_file
+        self.scs.remote_dir = remote_dir
+        self.scs.remote_file = remote_file
+        return remote_file, remote_dir
 
-    def expand_remote_path(self):
-        """expands the remote path to its absolute path
-        :return None:
+    def expand_remote_path(self, remote_path):
+        """if only a filename is provided, expands the remote
+        path to its absolute path
+        :param remote_path:
+        :type string:
+        :return remote_path:
+        :type string:
         :raises ValueError: if remote cmd fails
         """
         logger.info("entering expand_remote_path()")
-        if not self.remote_path or re.match(r"\.", self.remote_path):
-            result, stdout = self.sshshell.run("pwd")
+        if (
+            not re.search(r"\/", remote_path)
+            or re.match(r"\.\/", remote_path)
+            or not remote_path
+        ):
+            result, stdout = self.scs.ssh_cmd("pwd")
             if result:
-                pwd = stdout.split("\n")[1].rstrip()
-                self.remote_path = f"{pwd}{self.remote_path.lstrip('.')}"
-                logger.debug(f"remote_path now = {self.remote_path}")
+                if self.use_shell:
+                    pwd = stdout.split("\n")[1].rstrip()
+                else:
+                    pwd = stdout
+                remote_path = re.sub(r"^\.\/", "", remote_path)
+                remote_path = f"{pwd}/{remote_path}"
+                logger.debug(f"remote_path now = {remote_path}")
             else:
                 raise ValueError(
                     "Cannot determine the current working directory on the remote host"
                 )
+        return remote_path
 
-    def path_startswith_tilda(self):
+    def path_startswith_tilda(self, remote_path):
         """expands ~ based path to absolute path
-        :return None:
+        :param remote_path:
+        :type string:
+        :return remote_path:
+        :type string:
         :raises ValueError: if remote cmd fails
         """
         logger.info("entering path_startswith_tilda()")
-        if re.match(r"~", self.remote_path):
-            result, stdout = self.sshshell.run(f"ls -d {self.remote_path}")
+        if re.match(r"~", remote_path):
+            result, stdout = self.scs.ssh_cmd(f"ls -d {remote_path}")
             if result:
-                self.remote_path = stdout.split("\n")[1].rstrip()
-                logger.debug(f"remote_path now = {self.remote_path}")
+                if self.use_shell:
+                    remote_path = stdout.split("\n")[1].rstrip()
+                else:
+                    remote_path = stdout
+                logger.debug(f"remote_path now = {remote_path}")
             else:
-                raise ValueError(f"unable to expand remote path {self.remote_path}")
+                raise ValueError(f"unable to expand remote path {remote_path}")
+        return remote_path
 
-    def check_target_exists(self):
+    def check_target_exists(self, remote_dir, remote_file):
         """checks if the target file already exists
+        :param remote_dir:
+        :type string:
+        :param remote_file:
+        :type string:
         :return result:
         :type bool:
         """
         logger.info("entering check_target_exists()")
-        result, stdout = self.sshshell.run(
-            f"test -e {self.remote_dir}/{self.remote_file}"
-        )
+        result, stdout = self.scs.ssh_cmd(f"test -e {remote_dir}/{remote_file}")
         return result
 
-    def delete_target_remote(self):
-        """attempts to delete the target file
+    def delete_target_remote(self, remote_dir, remote_file):
+        """verifies whether path is a file.
+        if true and --overwrite flag is specified attempt to delete it
+        else alert the user and exit
+        :param remote_dir:
+        :type string:
+        :param remote_file:
+        :type string:
         :return None:
         """
         logger.info("entering delete_target_remote()")
-        result, stdout = self.sshshell.run(
-            f"rm -f {self.remote_dir}/{self.remote_file}"
-        )
-        if not result:
-            err = "remote file already exists, and could not be deleted"
-            self.scs.close(err_str=err)
+        if self.check_target_exists(remote_dir, remote_file):
+            if self.overwrite:
+                result, stdout = self.scs.ssh_cmd(f"rm -f {remote_dir}/{remote_file}")
+                if not result:
+                    err = "remote file already exists, and could not be deleted"
+                    self.scs.close(err_str=err)
+            else:
+                err = "remote file already exists. use --overwrite arg or delete it manually"
+                self.scs.close(err_str=err)
 
     def determine_local_filesize(self):
         """determines the local files size in bytes
@@ -336,6 +372,9 @@ class SplitCopyPut:
         logger.info("entering determine_local_filesize()")
         file_size = os.path.getsize(self.local_path)
         logger.info(f"src file size is {file_size}")
+        if not file_size:
+            err = "local file size is 0 bytes, nothing to copy"
+            self.scs.close(err_str=err)
         return file_size
 
     def local_sha_put(self):
@@ -383,6 +422,10 @@ class SplitCopyPut:
     def split_file_local(self, file_size, split_size):
         """splits file into chunks of size already determined in file_split_size()
         This function emulates GNU split.
+        :param file_size:
+        :type int:
+        :param split_size:
+        :type int:
         :returns None:
         """
         print("splitting file...")
@@ -405,10 +448,22 @@ class SplitCopyPut:
                         sfx_2 = chr(ord(sfx_2) + 1)
         except Exception as error:
             err = f"an error occurred while splitting the file, the error was:\n{error}"
-            self.scs.close(err_str=err, hard_close=self.hard_close)
+            self.scs.close(err_str=err)
 
-    def join_files_remote(self, scp_lib, chunks, remote_tmpdir):
+    def join_files_remote(
+        self, scp_lib, chunks, remote_dir, remote_file, remote_tmpdir
+    ):
         """concatenates the files chunks into one file on remote host
+        :param scp_lib:
+        :type class:
+        :param chunks:
+        :type list:
+        :param remote_dir:
+        :type string:
+        :param remote_file:
+        :type string:
+        :param remote_tmpdir:
+        :type string:
         :returns None:
         """
         logger.info("entering join_files_remote()")
@@ -419,7 +474,7 @@ class SplitCopyPut:
             for chunk in chunks:
                 cmd += (
                     f"cat {remote_tmpdir}/{chunk[0]} "
-                    f">>{self.remote_dir}/{self.remote_file}\n"
+                    f">>{remote_dir}/{remote_file}\n"
                     "if [ $? -gt 0 ]; then exit 1; fi\n"
                     f"rm {remote_tmpdir}/{chunk[0]}\n"
                     "if [ $? -gt 0 ]; then exit 1; fi\n"
@@ -430,8 +485,10 @@ class SplitCopyPut:
                 transport = self.sshshell._transport
                 with scp_lib(transport) as scpclient:
                     scpclient.put("join.sh", f"{remote_tmpdir}/join.sh")
-            result, stdout = self.sshshell.run(
-                f"sh {remote_tmpdir}/join.sh", timeout=600
+            result, stdout = self.scs.ssh_cmd(
+                f"sh {remote_tmpdir}/join.sh",
+                timeout=600,
+                retry=False,
             )
         except Exception as err:
             logger.debug("".join(traceback.format_exception(*sys.exc_info())))
@@ -440,7 +497,6 @@ class SplitCopyPut:
                     f"{err.__class__.__name__} while combining file chunks on "
                     f"remote host: {str(err)}"
                 ),
-                hard_close=self.hard_close,
             )
 
         if not result:
@@ -448,42 +504,44 @@ class SplitCopyPut:
                 err_str=(
                     "failed to combine chunks on remote host. " f"error was:\n{stdout}"
                 ),
-                hard_close=self.hard_close,
             )
+        return result
 
-    def compare_file_sizes(self, file_size):
+    def compare_file_sizes(self, file_size, remote_dir, remote_file):
         """obtains the newly combined file size, compares it to the source files size
         :param file_size:
         :type int:
+        :param remote_dir:
+        :type string:
+        :param remote_file:
+        :type string:
         :return None:
         """
         logger.info("entering compare_file_sizes()")
-        result, stdout = self.sshshell.run(
-            f"ls -l {self.remote_dir}/{self.remote_file}"
-        )
+        result, stdout = self.scs.ssh_cmd(f"ls -l {remote_dir}/{remote_file}")
         if not result:
             self.scs.close(
                 err_str=(
-                    f"file {self.host}:{self.remote_dir}/{self.remote_file} "
-                    "not found! please retry"
+                    f"file {self.host}:{remote_dir}/{remote_file} not found! please retry"
                 ),
                 config_rollback=False,
-                hard_close=self.hard_close,
             )
-        combined_file_size = int(stdout.split("\r\n")[1].split()[4])
+        if self.use_shell:
+            combined_file_size = int(stdout.split("\r\n")[1].split()[4])
+        else:
+            combined_file_size = int(stdout.split()[4])
         if combined_file_size != file_size:
             self.scs.close(
                 err_str=(
                     f"combined file size is {combined_file_size}, file "
-                    f"{self.host}:{self.remote_dir}/{self.remote_file} size "
+                    f"{self.host}:{remote_dir}/{remote_file} size "
                     f"is {file_size}. Unexpected mismatch in file size. Please retry"
                 ),
                 config_rollback=False,
-                hard_close=self.hard_close,
             )
         print("local and remote file sizes match")
 
-    def remote_sha_put(self, sha_bin, sha_len, sha_hash):
+    def remote_sha_put(self, sha_bin, sha_len, sha_hash, remote_dir, remote_file):
         """creates a sha hash for the newly combined file
         on the remote host compares against local sha
         :param sha_bin:
@@ -492,6 +550,10 @@ class SplitCopyPut:
         :type int:
         :param sha_hash:
         :type hash:
+        :param remote_dir:
+        :type string:
+        :param remote_file:
+        :type string:
         :returns None:
         """
         print("generating remote sha hash...")
@@ -501,13 +563,13 @@ class SplitCopyPut:
         else:
             cmd = f"{sha_bin}"
 
-        result, stdout = self.sshshell.run(
-            f"{cmd} {self.remote_dir}/{self.remote_file}", timeout=300
+        result, stdout = self.scs.ssh_cmd(
+            f"{cmd} {remote_dir}/{remote_file}", timeout=300
         )
         if not result:
             print(
                 "remote sha hash generation failed or timed out, "
-                f'manually check the output of "{cmd} {self.remote_dir}/{self.remote_file}" and '
+                f'manually check the output of "{cmd} {remote_dir}/{remote_file}" and '
                 f"compare against {sha_hash[sha_len]}"
             )
             return
@@ -521,22 +583,20 @@ class SplitCopyPut:
             self.scs.close(
                 err_str="failed to obtain remote sha hash to compare against",
                 config_rollback=False,
-                hard_close=self.hard_close,
             )
         logger.info(f"remote sha = {remote_sha}")
         if sha_hash[sha_len] == remote_sha:
             print(
                 f"local and remote sha hash match\nfile has been "
-                f"successfully copied to {self.host}:{self.remote_dir}/{self.remote_file}"
+                f"successfully copied to {self.host}:{remote_dir}/{remote_file}"
             )
         else:
             self.scs.close(
                 err_str=(
-                    f"file has been copied to {self.host}:{self.remote_dir}/{self.remote_file}"
+                    f"file has been copied to {self.host}:{remote_dir}/{remote_file}"
                     ", but the local and remote sha do not match - please retry"
                 ),
                 config_rollback=False,
-                hard_close=self.hard_close,
             )
 
     def put_files(self, ftp_lib, ssh_lib, scp_lib, chunk, remote_tmpdir, ssh_kwargs):
