@@ -58,7 +58,7 @@ class SplitCopyShared:
         self.ssh_key = kwargs.get("ssh_key")
         self.ssh_port = kwargs.get("ssh_port")
         self.local_dir = kwargs.get("local_dir")
-        self.get_op = kwargs.get("get")
+        self.copy_op = kwargs.get("copy_op")
         self.remote_dir = ""
         self.remote_file = ""
         self.command_list = []
@@ -66,6 +66,8 @@ class SplitCopyShared:
         self.local_tmpdir = None
         self.remote_tmpdir = None
         self.sshshell = None
+        self.use_shell = False
+        self.hard_close = False
 
     def connect(self, ssh_lib, **ssh_kwargs):
         """open an ssh session to a remote host
@@ -83,16 +85,12 @@ class SplitCopyShared:
             self.sshshell = ssh_lib(**ssh_kwargs)
             self.sshshell.socket_open()
             self.sshshell.transport_open()
+            self.sshshell.set_transport_keepalive()
             if self.sshshell.main_thread_auth():
-                self.sshshell.channel_open()
-                self.sshshell.invoke_shell()
                 ssh_kwargs = self.sshshell.kwargs
                 logger.debug(f"ssh_kwargs returned are: {ssh_kwargs}")
             else:
                 raise SSHException("authentication failed")
-            self.sshshell.set_keepalive()
-            # remove the welcome message from the socket
-            self.sshshell.stdout_read(timeout=30)
         except Exception as err:
             logger.debug("".join(traceback.format_exception(*sys.exc_info())))
             if self.sshshell is not None:
@@ -175,8 +173,24 @@ class SplitCopyShared:
             result = True
         return result
 
+    def juniper_cli_check(self):
+        """determines whether exec cmd is run on a juniper cli
+        :returns bool:
+        """
+        logger.info("entering juniper_cli_check()")
+        result, stdout = self.ssh_cmd("uname")
+        if result and stdout == "\nerror: unknown command: uname":
+            # this is junos or evo CLI. exit code is always 0.
+            self.use_shell = True
+        elif result:
+            pass
+        else:
+            err = "cmd 'uname' failed on remote host, it must be *nix based"
+            self.close(err_str=err)
+        return self.use_shell
+
     def which_os(self):
-        """determines if host is JUNOS/EVO/*nix
+        """determines if host is JUNOS or EVO
         no support for remote Windows OS running OpenSSH
         :return junos:
         :type bool:
@@ -192,18 +206,28 @@ class SplitCopyShared:
         evo = False
         bsd_version = float()
         sshd_version = float()
-        self.sshshell.run("start shell", exitcode=False)
-        result, stdout = self.sshshell.run("uname")
+        result, stdout = self.ssh_cmd("uname")
         if not result:
-            err = "failed to determine remote host os, it must be *nix based"
+            err = "cmd 'uname' failed on remote host, it must be *nix based"
             self.close(err_str=err)
-        host_os = stdout.split("\n")[1].rstrip()
+        if self.use_shell:
+            host_os = stdout.split("\n")[1].rstrip()
+        else:
+            host_os = stdout
         if host_os == "Linux" and self.evo_os():
             evo = True
-        else:
-            junos, bsd_version, sshd_version = self.junos_os()
+        elif host_os == "JUNOS":
+            junos = True
+            bsd_version = 6.3
+            sshd_version = self.which_sshd()
+        elif host_os == "FreeBSD" and self.junos_os():
+            junos = True
+            bsd_version = self.which_bsd()
+            sshd_version = self.which_sshd()
         logger.info(
-            f"evo = {evo}, junos = {junos}, bsd_version = {bsd_version}, "
+            f"evo = {evo}, "
+            f"junos = {junos}, "
+            f"bsd_version = {bsd_version}, "
             f"sshd_version = {sshd_version}"
         )
         return junos, evo, bsd_version, sshd_version
@@ -214,38 +238,17 @@ class SplitCopyShared:
         :type bool:
         """
         logger.info("entering evo_os()")
-        result, stdout = self.sshshell.run("test -e /usr/sbin/evo-pfemand")
+        result, stdout = self.ssh_cmd("test -e /usr/sbin/evo-pfemand")
         return result
 
     def junos_os(self):
         """determines if host is running JUNOS
-        and if so which bsd and sshd versions are in use
-        :return junos:
+        :return result:
         :type bool:
-        :return bsd_version:
-        :type float:
-        :return sshd_version:
-        :type float:
         """
         logger.info("entering junos_os()")
-        junos = False
-        bsd_version = float()
-        sshd_version = float()
-        result, stdout = self.sshshell.run("uname -i")
-        if not result:
-            self.close(err_str="failed to determine remote host os")
-        uname = stdout.split("\n")[1]
-        if re.match(r"JUNIPER", uname):
-            junos = True
-            bsd_version = 6.0
-            sshd_version = self.which_sshd()
-        elif re.match(r"JNPR", uname):
-            junos = True
-            bsd_version = self.which_bsd()
-            sshd_version = self.which_sshd()
-        else:
-            sshd_version = self.which_sshd()
-        return junos, bsd_version, sshd_version
+        result, stdout = self.ssh_cmd("uname -i | egrep 'JUNIPER|JNPR'")
+        return result
 
     def which_bsd(self):
         """determines the BSD version of JUNOS
@@ -253,10 +256,13 @@ class SplitCopyShared:
         :type float:
         """
         logger.info("entering which_bsd()")
-        result, stdout = self.sshshell.run("uname -r")
+        result, stdout = self.ssh_cmd("uname -r")
         if not result:
             self.close(err_str="failed to determine remote bsd version")
-        uname = stdout.split("\n")[1]
+        if self.use_shell:
+            uname = stdout.split("\n")[1]
+        else:
+            uname = stdout
         bsd_version = float(uname.split("-")[1])
         return bsd_version
 
@@ -266,18 +272,21 @@ class SplitCopyShared:
         :type float
         """
         logger.info("entering which_sshd()")
-        result, stdout = self.sshshell.run("sshd -v", exitcode=False)
-        if not re.search(r"OpenSSH_", stdout):
-            self.close(err_str="failed to determine remote openssh version")
-        output = stdout.split("\n")[2]
+        result, stdout = self.ssh_cmd("sshd -v", exitcode=False, combine=True)
+        if self.use_shell:
+            if not re.search(r"OpenSSH_", stdout):
+                self.close(err_str="failed to determine remote openssh version")
+            output = stdout.split("\n")[2]
+        else:
+            if not re.search(r"OpenSSH_", stdout):
+                self.close(err_str="failed to determine remote openssh version")
+            output = stdout.split("\n")[1]
         version = re.sub(r"OpenSSH_", "", output)
         sshd_version = float(version[0:3])
         return sshd_version
 
-    def req_binaries(self, get_op=False, junos=False, evo=False):
+    def req_binaries(self, junos=False, evo=False):
         """ensures required binaries exist on remote host
-        :param get_op:
-        :type bool:
         :param junos:
         :type bool:
         :param evo:
@@ -286,11 +295,11 @@ class SplitCopyShared:
         """
         logger.info("entering req_binaries()")
         if not junos and not evo:
-            if get_op:
+            if self.copy_op == "get":
                 req_bins = "dd ls df rm"
             else:
                 req_bins = "cat ls df rm"
-            result, stdout = self.sshshell.run(f"which {req_bins}")
+            result, stdout = self.ssh_cmd(f"which {req_bins}")
             if not result:
                 self.close(
                     err_str=(
@@ -331,7 +340,7 @@ class SplitCopyShared:
         logger.info(sha_bins)
 
         for req_bin in sha_bins:
-            result, stdout = self.sshshell.run(f"which {req_bin[0]}")
+            result, stdout = self.ssh_cmd(f"which {req_bin[0]}")
             if result:
                 sha_bin = req_bin[0]
                 sha_len = req_bin[1]
@@ -345,7 +354,7 @@ class SplitCopyShared:
             )
         return sha_bin, sha_len
 
-    def close(self, err_str=None, config_rollback=True, hard_close=False):
+    def close(self, err_str=None, config_rollback=True):
         """called when we want to exit the script
         attempts to delete the remote temp directory and close the TCP session
         If hard_close == False, contextmanager will rm the local temp dir
@@ -354,21 +363,27 @@ class SplitCopyShared:
         :type: string:
         :param config_rollback:
         :type bool:
-        :param hard_close:
-        :type bool:
         :raises SystemExit: terminates the script gracefully
         :raises os._exit: terminates the script immediately (even asyncio loop)
         """
         logger.info("entering close()")
         if err_str:
             print(err_str)
-        if self.rm_remote_tmp:
-            self.remote_cleanup()
-        if config_rollback and self.command_list:
-            self.limits_rollback()
-        print(f"\r{pad_string('closing device connection')}")
-        self.sshshell.close()
-        if hard_close:
+        if (
+            self.use_shell
+            and self.sshshell._chan is not None
+            and not self.sshshell._chan.closed
+            or not self.use_shell
+            and self.sshshell._transport is not None
+            and self.sshshell._transport.active
+        ):
+            if self.rm_remote_tmp:
+                self.remote_cleanup()
+            if config_rollback and self.command_list:
+                self.limits_rollback()
+            print(f"\r{pad_string('closing device connection')}")
+            self.sshshell.close()
+        if self.hard_close:
             try:
                 shutil.rmtree(self.local_tmpdir)
             except PermissionError:
@@ -412,11 +427,11 @@ class SplitCopyShared:
             # +1 user process if openssh version is >= 7.4
             pid_count = 0
             max_pids = 40
-            if sshd_version >= 7.4 and bsd_version == 6.0:
+            if sshd_version >= 7.4 and bsd_version == 6.3:
                 pid_count = 4
             elif sshd_version >= 7.4 and bsd_version >= 10.0:
                 pid_count = 3
-            elif bsd_version == 6.0:
+            elif bsd_version == 6.3:
                 pid_count = 3
             elif bsd_version >= 10.0:
                 pid_count = 2
@@ -445,64 +460,79 @@ class SplitCopyShared:
         )
         return split_size, executor
 
-    def mkdir_remote(self):
+    def mkdir_remote(self, remote_dir, remote_file):
         """creates a tmp directory on the remote host
+        :param remote_dir:
+        :type string:
+        :param remote_file:
+        :type string:
         :returns remote_tmpdir:
         :type string:
         """
         logger.info("entering mkdir_remote()")
         time_stamp = datetime.datetime.strftime(datetime.datetime.now(), "%y%m%d%H%M%S")
-        if self.get_op:
-            remote_tmpdir = f"/var/tmp/splitcopy_{self.remote_file}.{time_stamp}"
-        else:
-            remote_tmpdir = (
-                f"{self.remote_dir}/splitcopy_{self.remote_file}.{time_stamp}"
-            )
-        result, stdout = self.sshshell.run(f"mkdir -p {remote_tmpdir}")
+        self.remote_tmpdir = f"{remote_dir}/splitcopy_{remote_file}.{time_stamp}"
+        result, stdout = self.ssh_cmd(f"mkdir -p {self.remote_tmpdir}")
         if not result:
-            err = (
-                "unable to create the tmp directory on remote host."
-                f"cmd output was:\n{stdout}"
-            )
+            err = f"unable to create the tmp directory {self.remote_tmpdir} on remote host"
             self.close(err_str=err)
         self.rm_remote_tmp = True
-        return remote_tmpdir
+        return self.remote_tmpdir
 
-    def storage_check_remote(self, file_size, split_size):
+    def storage_check_remote(self, file_size, split_size, remote_dir):
         """checks whether there is enough storage space on remote node
         :param file_size:
         :type int:
         :param split_size:
         :type int:
+        :param remote_dir:
+        :type string:
         :returns None:
         """
         logger.info("entering storage_check_remote()")
         avail_blocks = 0
         print("checking remote storage...")
-        result, stdout = self.sshshell.run(f"df -k {self.remote_dir}")
+        result, stdout = self.ssh_cmd(f"df -k {remote_dir}")
         if not result:
             self.close(err_str="failed to determine remote disk space available")
         try:
-            avail_blocks = re.search(r" ([0-9]+) +[0-9]+ +[0-9]+ +", stdout).group(1)
+            fs_blocks = re.search(r" ([0-9]+) +([0-9]+) +(-?[0-9]+) +", stdout)
+            total_blocks = int(fs_blocks.group(1))
+            used_blocks = int(fs_blocks.group(2))
+            avail_blocks = int(fs_blocks.group(3))
         except AttributeError:
-            err = "unable to determine available blocks on remote host"
+            err = "unable to determine available storage on remote host"
             self.close(err_str=err)
 
-        avail_bytes = int(avail_blocks) * 1024
+        if avail_blocks < 0:
+            reserved_blocks_percent = 100 - round(
+                100 / total_blocks * (used_blocks + avail_blocks)
+            )
+            reserved_blocks_threshold = used_blocks + avail_blocks
+            reserved_blocks_count = total_blocks - (used_blocks + avail_blocks)
+            err = (
+                f"not enough available storage on remote host in {remote_dir}\n"
+                f"{reserved_blocks_count} / {reserved_blocks_percent}% of 1024-byte blocks "
+                "are reserved and may only be allocated by privileged processes\n"
+                f"used blocks: {used_blocks} is > than the threshold for reserved blocks: {reserved_blocks_threshold}"
+            )
+            self.close(err_str=err)
+
+        avail_bytes = avail_blocks * 1024
         logger.info(f"remote filesystem available bytes is {avail_bytes}")
-        if self.get_op:
+        if self.copy_op == "get":
             if file_size > avail_bytes:
                 err = (
-                    "not enough storage on remote host in /var/tmp.\nAvailable bytes "
-                    f"({avail_bytes}) must be > the original file size "
+                    f"not enough storage on remote host in {remote_dir}\n"
+                    f"available bytes ({avail_bytes}) must be >= the original file size "
                     f"({file_size}) because it has to store the file chunks"
                 )
                 self.close(err_str=err)
         else:
             if file_size + split_size > avail_bytes:
                 err = (
-                    f"not enough storage on remote host in {self.remote_dir}.\n"
-                    f"Available bytes ({avail_bytes}) must be > "
+                    f"not enough storage on remote host in {remote_dir}\n"
+                    f"available bytes ({avail_bytes}) must be > "
                     f"the original file size ({file_size}) + largest chunk size "
                     f"({split_size})"
                 )
@@ -527,7 +557,7 @@ class SplitCopyShared:
             )
             self.close(err_str=err)
 
-        if self.get_op:
+        if self.copy_op == "get":
             avail_bytes = shutil.disk_usage(self.local_dir)[2]
             logger.info(
                 f"local filesystem {self.local_dir} available bytes is {avail_bytes}"
@@ -580,19 +610,25 @@ class SplitCopyShared:
         """
         return self.local_tmpdir
 
-    def find_configured_limits(self, config_stanzas):
+    def find_configured_limits(self, config_stanzas, limits):
         """Function that retrieves any configuration stazas that implement
         rate/connection limits.
+        It is faster to perform grep on the router, than to transfer
+        potentially huge amounts of text and do it locally
         :param config_stanzas:
         :type list:
+        :param limits:
+        :type list:
         :return cli_config:
         :type string:
         """
         logger.info("entering find_configured_limits()")
         cli_config = ""
+        limits_str = "|".join(limits)
         for stanza in config_stanzas:
-            result, stdout = self.sshshell.run(
-                f'cli -c "show configuration {stanza} | display set | no-more"'
+            result, stdout = self.ssh_cmd(
+                f"cli -c 'show configuration {stanza} | display set | "
+                f'grep "{limits_str}" | no-more\'',
             )
             cli_config += stdout
         return cli_config
@@ -613,7 +649,8 @@ class SplitCopyShared:
         if copy_proto == "ftp":
             limits.append("services ftp connection-limit")
             limits.append("services ftp rate-limit")
-        cli_config = self.find_configured_limits(config_stanzas)
+        print("checking router configuration... ")
+        cli_config = self.find_configured_limits(config_stanzas, limits)
 
         for limit in limits:
             re_limit_multiline = re.compile(rf"^set .*{limit} [0-9]", re.MULTILINE)
@@ -633,7 +670,7 @@ class SplitCopyShared:
         if self.command_list:
             print("rate-limit/connection-limit/login retry-options configuration found")
             logger.info(self.command_list)
-            result, stdout = self.sshshell.run(
+            result, stdout = self.ssh_cmd(
                 f'cli -c "edit;{"".join(self.command_list)}commit and-quit"',
                 exitcode=False,
                 timeout=60,
@@ -641,7 +678,7 @@ class SplitCopyShared:
             # cli always returns true so can't use exitcode
             if re.search(r"commit complete\r\nExiting configuration mode", stdout):
                 print("configuration has been modified. deactivated the relevant lines")
-                self.sshshell.run(
+                self.ssh_cmd(
                     "logger 'splitcopy has made the following config changes: "
                     f"{''.join(self.command_list)}'",
                     exitcode=False,
@@ -661,7 +698,7 @@ class SplitCopyShared:
         logger.info("entering limits_rollback()")
         rollback_cmds = "".join(self.command_list)
         rollback_cmds = re.sub("deactivate", "activate", rollback_cmds)
-        result, stdout = self.sshshell.run(
+        result, stdout = self.ssh_cmd(
             f'cli -c "edit;{rollback_cmds}commit and-quit"',
             exitcode=False,
             timeout=60,
@@ -669,7 +706,7 @@ class SplitCopyShared:
         # cli always returns true so can't use exitcode
         if re.search(r"commit complete\r\nExiting configuration mode", stdout):
             print("configuration changes made have been reverted")
-            self.sshshell.run(
+            self.ssh_cmd(
                 "logger 'splitcopy has reverted config changes'",
                 exitcode=False,
             )
@@ -690,6 +727,7 @@ class SplitCopyShared:
         :return None:
         """
         logger.info("entering remote_cleanup()")
+        result = False
         if remote_dir:
             self.remote_dir = remote_dir
         if remote_file:
@@ -697,17 +735,90 @@ class SplitCopyShared:
         if not silent:
             print(f"\r{pad_string('deleting remote tmp directory...')}")
         if self.remote_tmpdir is None:
-            if self.get_op:
-                self.sshshell.run(f"rm -rf /var/tmp/splitcopy_{self.remote_file}.*")
+            if self.copy_op == "get":
+                result, stdout = self.ssh_cmd(
+                    f"rm -rf /var/tmp/splitcopy_{self.remote_file}.*"
+                )
             else:
-                self.sshshell.run(
+                result, stdout = self.ssh_cmd(
                     f"rm -rf {self.remote_dir}/splitcopy_{self.remote_file}.*"
                 )
         else:
-            result, stdout = self.sshshell.run(f"rm -rf {self.remote_tmpdir}")
+            result, stdout = self.ssh_cmd(f"rm -rf {self.remote_tmpdir}")
             if not result and not silent:
                 print(
                     f"unable to delete the tmp directory {self.remote_tmpdir} on remote host, "
                     "delete it manually"
                 )
         self.rm_remote_tmp = False
+        return result
+
+    def enter_shell(self):
+        """in order to drop into shell from cli mode, a pty and interactive shell are required
+        :return result:
+        :type bool:
+        """
+        try:
+            # request a channel
+            self.sshshell.channel_open()
+            # request a pty and an interactive shell session
+            self.sshshell.invoke_shell()
+            # remove the welcome message from the socket
+            self.sshshell.stdout_read(timeout=30)
+        except SSHException as err:
+            self.close(err_str=err)
+        # enter shell mode
+        result, stdout = self.ssh_cmd("start shell", exitcode=False)
+        return result
+
+    def ssh_cmd(
+        self, cmd, timeout=30, exitcode=True, combine=False, retry=True, count=0
+    ):
+        """wrapper around functions that send a cmd to a remote host.
+        which function gets called depends on whether an interactive shell is in use.
+        if exitcode is True will check its exit status
+        :param cmd: cmd to run on remote host
+        :type string:
+        :param timeout: amount of time before timeout is raised
+        :type float:
+        :param exitcode: toggles whether to check for exit status or not
+        :type bool:
+        :return result: whether successful or not
+        :type bool:
+        :return stdout: the output of the command
+        :type string:
+        """
+        result = False
+        stdout = ""
+        logger.debug(cmd)
+        try:
+            if self.use_shell:
+                result, stdout = self.sshshell.shell_cmd(cmd, timeout, exitcode)
+            else:
+                result, stdout = self.sshshell.exec_cmd(cmd, timeout, combine)
+            logger.debug(result)
+        except TimeoutError:
+            count += 1
+            timeout = timeout * 2
+            if self.use_shell:
+                # channel is now unusable, close it and open a new channel
+                self.sshshell.close_channel()
+                self.enter_shell()
+            if retry:
+                if count == 3:
+                    self.close(err_str="cmd timed out")
+                print(
+                    f"cmd '{cmd}' timed out, retrying with a timeout of {timeout} secs"
+                )
+                result, stdout = self.ssh_cmd(
+                    cmd,
+                    timeout=timeout,
+                    exitcode=exitcode,
+                    combine=combine,
+                    count=count,
+                )
+        except SSHException as err:
+            self.close(err_str=f"ssh exception '{err}' raised while running '{cmd}'")
+        except OSError as err:
+            self.close(err_str=f"OSError exception raised while running '{cmd}'")
+        return result, stdout
