@@ -124,6 +124,9 @@ class MockSplitCopyShared:
     def __init__(self):
         pass
 
+    def juniper_cli_check(*args):
+        return True
+
     def close(self, **kwargs):
         raise SystemExit
 
@@ -154,7 +157,7 @@ class MockSplitCopyShared:
     def storage_check_local(self, *args):
         pass
 
-    def mkdir_remote(self):
+    def mkdir_remote(self, *args):
         return "/var/tmp/foo"
 
     def limit_check(self, *args):
@@ -167,6 +170,12 @@ class MockSplitCopyShared:
     def tempdir(*args):
         yield
 
+    def ssh_cmd(*args, **kwargs):
+        return (True, "")
+
+    def enter_shell(self):
+        return True
+
 
 class MockHash:
     def __init__(self):
@@ -191,25 +200,22 @@ class TestSplitCopyPut:
         scput.scs = MockSplitCopyShared()
         scput.progress = MockProgress()
 
-        def validate_remote_path_put():
+        def validate_remote_path_put(*args):
+            return ("", "")
+
+        def delete_target_remote(*args):
             pass
 
-        def check_target_exists():
-            return True
-
-        def delete_target_remote():
-            pass
-
-        def determine_local_filesize():
+        def determine_local_filesize(*args):
             return 1000000
 
         def local_sha_put():
             return "sha384sum", 384, "abcdef0123456789"
 
-        def split_file_local(file_size, split_size):
+        def split_file_local(*args):
             pass
 
-        def get_chunk_info():
+        def get_chunk_info(*args):
             return [["chunk0", 1000], ["chunk1", 1000]]
 
         def put_files(*args):
@@ -230,7 +236,6 @@ class TestSplitCopyPut:
                 scput.progress.totals["percent_done"] = n
 
         monkeypatch.setattr(scput, "validate_remote_path_put", validate_remote_path_put)
-        monkeypatch.setattr(scput, "check_target_exists", check_target_exists)
         monkeypatch.setattr(scput, "delete_target_remote", delete_target_remote)
         monkeypatch.setattr(scput, "determine_local_filesize", determine_local_filesize)
         monkeypatch.setattr(scput, "local_sha_put", local_sha_put)
@@ -256,16 +261,13 @@ class TestSplitCopyPut:
         scput.scs = MockSplitCopyShared()
         scput.progress = MockProgress()
 
-        def validate_remote_path_put():
+        def validate_remote_path_put(*args):
+            return ("", "")
+
+        def delete_target_remote(*args):
             pass
 
-        def check_target_exists():
-            return True
-
-        def delete_target_remote():
-            pass
-
-        def determine_local_filesize():
+        def determine_local_filesize(*args):
             return 1000000
 
         def split_file_local(*args):
@@ -290,7 +292,6 @@ class TestSplitCopyPut:
 
         scput.noverify = True
         monkeypatch.setattr(scput, "validate_remote_path_put", validate_remote_path_put)
-        monkeypatch.setattr(scput, "check_target_exists", check_target_exists)
         monkeypatch.setattr(scput, "delete_target_remote", delete_target_remote)
         monkeypatch.setattr(scput, "determine_local_filesize", determine_local_filesize)
         monkeypatch.setattr(scput, "split_file_local", split_file_local)
@@ -314,16 +315,13 @@ class TestSplitCopyPut:
         scput.scs = MockSplitCopyShared()
         scput.progress = MockProgress()
 
-        def validate_remote_path_put():
+        def validate_remote_path_put(*args):
+            return ("", "")
+
+        def delete_target_remote(*args):
             pass
 
-        def check_target_exists():
-            return True
-
-        def delete_target_remote():
-            pass
-
-        def determine_local_filesize():
+        def determine_local_filesize(*args):
             return 1000000
 
         def local_sha_put():
@@ -339,7 +337,6 @@ class TestSplitCopyPut:
             raise TransferError
 
         monkeypatch.setattr(scput, "validate_remote_path_put", validate_remote_path_put)
-        monkeypatch.setattr(scput, "check_target_exists", check_target_exists)
         monkeypatch.setattr(scput, "delete_target_remote", delete_target_remote)
         monkeypatch.setattr(scput, "determine_local_filesize", determine_local_filesize)
         monkeypatch.setattr(scput, "local_sha_put", local_sha_put)
@@ -380,58 +377,48 @@ class TestSplitCopyPut:
     def test_validate_remote_path_put_no_remote_path(self, monkeypatch: MonkeyPatch):
         scput = SplitCopyPut()
 
-        class MockSSHShell2(MockSSHShell):
-            def run(cmd):
-                return (True, "")
+        def expand_remote_path(*args):
+            return "/homes/foobar"
 
-        def expand_remote_path():
-            scput.remote_path = "/homes/foobar"
+        def path_startswith_tilda(*args):
+            return "/homes/foobar"
 
-        def path_startswith_tilda():
-            pass
-
-        scput.sshshell = MockSSHShell2
         scput.scs = MockSplitCopyShared
         monkeypatch.setattr(scput, "expand_remote_path", expand_remote_path)
         monkeypatch.setattr(scput, "path_startswith_tilda", path_startswith_tilda)
         scput.local_file = "foo"
         scput.remote_path = ""
-        scput.validate_remote_path_put()
-        assert scput.remote_dir == "/homes/foobar" and scput.remote_file == "foo"
+        result = scput.validate_remote_path_put()
+        assert result == ("foo", "/homes/foobar")
 
     def test_validate_remote_path_put_isdir(self):
         scput = SplitCopyPut()
-
-        class MockSSHShell2(MockSSHShell):
-            def run(cmd):
-                return (True, "")
-
-        scput.sshshell = MockSSHShell2
+        scput.scs = MockSplitCopyShared
         scput.local_file = "foo"
         scput.remote_path = "/var/tmp"
-        scput.validate_remote_path_put()
-        assert scput.remote_dir == "/var/tmp" and scput.remote_file == "foo"
+        result = scput.validate_remote_path_put()
+        assert result == ("foo", "/var/tmp")
 
     def test_validate_remote_path_existingfile_basename_match(self):
-        class MockSSHShell2(MockSSHShell):
-            def run(cmd):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(cmd):
                 if cmd == "test -d /var/tmp/foo":
                     return False, ""
                 else:
                     return True, ""
 
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2
+        scput.scs = MockSplitCopyShared2
         scput.local_file = "foo"
         scput.remote_path = "/var/tmp/foo"
         result = scput.validate_remote_path_put()
-        assert result == None
+        assert result == ("foo", "/var/tmp")
 
     def test_validate_remote_path_put_newfile_basename_nomatch(self):
-        class MockSSHShell2(MockSSHShell):
+        class MockSplitCopyShared2(MockSplitCopyShared):
             self.instance = 0
 
-            def run(cmd):
+            def ssh_cmd(cmd):
                 if re.match(r"test -d", cmd) and not self.instance:
                     self.instance += 1
                     return False, ""
@@ -439,15 +426,15 @@ class TestSplitCopyPut:
                     return True, ""
 
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2
+        scput.scs = MockSplitCopyShared2
         scput.local_file = "foo"
         scput.remote_path = "/var/tmp/bar"
         result = scput.validate_remote_path_put()
-        assert result == None
+        assert result == ("bar", "/var/tmp")
 
     def test_validate_remote_path_put_fail(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(cmd):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
                 return (False, "")
 
         def dirname(path):
@@ -455,15 +442,14 @@ class TestSplitCopyPut:
 
         monkeypatch.setattr("os.path.dirname", dirname)
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2
-        scput.scs = MockSplitCopyShared()
+        scput.scs = MockSplitCopyShared2()
         scput.local_file = "foo"
         scput.remote_path = "/var/tmp/foo"
         with raises(SystemExit):
             scput.validate_remote_path_put()
 
     def test_validate_remote_path_put_fail2(self, monkeypatch: MonkeyPatch):
-        def expand_remote_path():
+        def expand_remote_path(*args):
             raise ValueError
 
         scput = SplitCopyPut()
@@ -472,92 +458,154 @@ class TestSplitCopyPut:
         with raises(SystemExit):
             scput.validate_remote_path_put()
 
-    def test_expand_remote_path(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(cmd):
-                stdout = "foo@bar ~ % pwd\n" "/homes/foo\n" "foo@bar ~ % \n"
+    def test_expand_remote_path_shell(self, monkeypatch: MonkeyPatch):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
+                stdout = "foo@bar ~ % pwd\n/homes/foo\nfoo@bar ~ % \n"
                 return (True, stdout)
 
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2
-        scput.remote_path = "./tmp"
-        scput.expand_remote_path()
-        assert scput.remote_path == "/homes/foo/tmp"
+        scput.use_shell = True
+        scput.scs = MockSplitCopyShared2()
+        result = scput.expand_remote_path("./tmp")
+        assert result == "/homes/foo/tmp"
+
+    def test_expand_remote_path_exec(self, monkeypatch: MonkeyPatch):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
+                stdout = "/homes/foo"
+                return (True, stdout)
+
+        scput = SplitCopyPut()
+        scput.scs = MockSplitCopyShared2()
+        result = scput.expand_remote_path("./tmp")
+        assert result == "/homes/foo/tmp"
 
     def test_expand_remote_path_fail(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(cmd):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
                 return (False, "")
 
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2
-        scput.remote_path = "./tmp"
+        scput.scs = MockSplitCopyShared2()
         with raises(ValueError):
-            scput.expand_remote_path()
+            scput.expand_remote_path("./tmp")
 
-    def test_path_startswith_tilda(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(cmd):
-                stdout = "foo@bar ~ % ls -d ~/tmp\n" "/homes/foo/tmp\n" "foo@bar ~ % \n"
+    def test_path_startswith_tilda_shell(self, monkeypatch: MonkeyPatch):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
+                stdout = "foo@bar ~ % ls -d ~/tmp\n/homes/foo/tmp\nfoo@bar ~ % \n"
                 return (True, stdout)
 
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2
-        scput.remote_path = "~/tmp"
-        scput.path_startswith_tilda()
-        assert scput.remote_path == "/homes/foo/tmp"
+        scput.use_shell = True
+        scput.scs = MockSplitCopyShared2()
+        result = scput.path_startswith_tilda("~/tmp")
+        assert result == "/homes/foo/tmp"
+
+    def test_path_startswith_tilda_exec(self, monkeypatch: MonkeyPatch):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
+                stdout = "/homes/foo/tmp"
+                return (True, stdout)
+
+        scput = SplitCopyPut()
+        scput.scs = MockSplitCopyShared2()
+        result = scput.path_startswith_tilda("~/tmp")
+        assert result == "/homes/foo/tmp"
 
     def test_path_startswith_tilda_fail(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(cmd):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
                 return (False, "")
 
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2
-        scput.remote_path = "~/tmp"
+        scput.scs = MockSplitCopyShared2()
         with raises(ValueError):
-            scput.path_startswith_tilda()
-
-    def test_check_target_exists(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(cmd):
-                return (True, "")
+            scput.path_startswith_tilda("~/tmp")
 
+    def test_check_target_exists(self):
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2
-        result = scput.check_target_exists()
+        scput.scs = MockSplitCopyShared()
+        remote_dir = "/var/tmp"
+        remote_file = "foobar"
+        result = scput.check_target_exists(remote_dir, remote_file)
         assert result == True
 
-    def test_delete_target_remote(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(cmd):
-                return (True, "")
-
-        scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2
-        result = scput.delete_target_remote()
-        assert result == None
-
-    def test_delete_target_remote_fail(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(cmd):
+    def test_check_target_exists_fail(self):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
                 return (False, "")
 
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2
+        scput.scs = MockSplitCopyShared2()
+        remote_dir = "/var/tmp"
+        remote_file = "foobar"
+        result = scput.check_target_exists(remote_dir, remote_file)
+        assert result == False
+
+    def test_delete_target_remote_no_overwrite(self, monkeypatch: MonkeyPatch):
+        def check_target_exists(*args):
+            return True
+
+        scput = SplitCopyPut()
+        monkeypatch.setattr(scput, "check_target_exists", check_target_exists)
         scput.scs = MockSplitCopyShared()
+        remote_dir = "/var/tmp"
+        remote_file = "foobar"
         with raises(SystemExit):
-            scput.delete_target_remote()
+            scput.delete_target_remote(remote_dir, remote_file)
+
+    def test_delete_target_remote_overwrite_fail(self, monkeypatch: MonkeyPatch):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
+                return (False, "")
+
+        def check_target_exists(*args):
+            return True
+
+        scput = SplitCopyPut()
+        scput.overwrite = True
+        monkeypatch.setattr(scput, "check_target_exists", check_target_exists)
+        scput.scs = MockSplitCopyShared2()
+        remote_dir = "/var/tmp"
+        remote_file = "foobar"
+        with raises(SystemExit):
+            scput.delete_target_remote(remote_dir, remote_file)
+
+    def test_delete_target_remote_overwrite_nofail(self, monkeypatch: MonkeyPatch):
+        def check_target_exists(*args):
+            return True
+
+        scput = SplitCopyPut()
+        scput.overwrite = True
+        monkeypatch.setattr(scput, "check_target_exists", check_target_exists)
+        scput.scs = MockSplitCopyShared()
+        remote_dir = "/var/tmp"
+        remote_file = "foobar"
+        result = scput.delete_target_remote(remote_dir, remote_file)
+        assert result == None
 
     def test_determine_local_filesize(self, monkeypatch: MonkeyPatch):
         def getsize(path):
             return 10000
 
         scput = SplitCopyPut()
+        scput.scs = MockSplitCopyShared()
         monkeypatch.setattr("os.path.getsize", getsize)
         result = scput.determine_local_filesize()
         assert result == 10000
 
+    def test_determine_local_filesize_fail(self, monkeypatch: MonkeyPatch):
+        def getsize(path):
+            return 0
+
+        scput = SplitCopyPut()
+        scput.scs = MockSplitCopyShared()
+        monkeypatch.setattr("os.path.getsize", getsize)
+        with raises(SystemExit):
+            scput.determine_local_filesize()
+
     def test_local_sha_put_512(self, monkeypatch: MonkeyPatch):
         def isfile(path):
             result = False
@@ -692,53 +740,66 @@ class TestSplitCopyPut:
             scput.split_file_local(file_size, split_size)
 
     def test_join_files_remote(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(self, cmd, timeout):
-                return True, ""
-
         chunks = [["somefile_aa", 10000], ["somefile_ab", 10000]]
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2()
         scput.scs = MockSplitCopyShared()
-        scput.remote_dir = "/var/tmp/foo"
-        scput.remote_file = "somefile"
+        scput.sshshell = MockSSHShell()
+        remote_dir = "/var/tmp/foo"
+        remote_file = "somefile"
+        remote_tmpdir = "/var/tmp/000"
         monkeypatch.setattr("builtins.open", MockOpen)
-        result = scput.join_files_remote(MockSCPClient, chunks, "/tmp/foo")
-        assert result == None
+        result = scput.join_files_remote(
+            MockSCPClient, chunks, remote_dir, remote_file, remote_tmpdir
+        )
+        assert result == True
 
     def test_join_files_remote_fail(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(self, cmd, timeout):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
                 return False, ""
 
         chunks = [["somefile_aa", 10000], ["somefile_ab", 10000]]
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2()
-        scput.scs = MockSplitCopyShared()
-        scput.remote_dir = "/var/tmp/foo"
-        scput.remote_file = "somefile"
+        scput.sshshell = MockSSHShell()
+        scput.scs = MockSplitCopyShared2()
+        remote_dir = "/var/tmp/foo"
+        remote_file = "somefile"
+        remote_tmpdir = "/var/tmp/000"
         monkeypatch.setattr("builtins.open", MockOpen)
         with raises(SystemExit):
-            scput.join_files_remote(MockSCPClient, chunks, "/tmp/foo")
+            scput.join_files_remote(
+                MockSCPClient,
+                chunks,
+                remote_dir,
+                remote_file,
+                remote_tmpdir,
+            )
 
     def test_join_files_remote_exception(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(self, cmd, timeout):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
                 raise SSHException
 
         chunks = [["somefile_aa", 10000], ["somefile_ab", 10000]]
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2()
-        scput.scs = MockSplitCopyShared()
-        scput.remote_dir = "/var/tmp/foo"
-        scput.remote_file = "somefile"
+        scput.sshshell = MockSSHShell()
+        scput.scs = MockSplitCopyShared2()
+        remote_dir = "/var/tmp/foo"
+        remote_file = "somefile"
+        remote_tmpdir = "/var/tmp/000"
         monkeypatch.setattr("builtins.open", MockOpen)
         with raises(SystemExit):
-            scput.join_files_remote(MockSCPClient, chunks, "/tmp/foo")
+            scput.join_files_remote(
+                MockSCPClient,
+                chunks,
+                remote_dir,
+                remote_file,
+                remote_tmpdir,
+            )
 
-    def test_compare_file_sizes(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(self, cmd, timeout=30):
+    def test_compare_file_sizes_shell(self, monkeypatch: MonkeyPatch):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
                 stdout = (
                     b"foo@bar ~ % ls -l /var/tmp/foo\r\r\n"
                     b"-rw-r--r--  1 foo  bar  100000 19 Dec  2019 /var/tmp/foo\r\n"
@@ -747,24 +808,41 @@ class TestSplitCopyPut:
                 return True, stdout
 
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2()
-        result = scput.compare_file_sizes(100000)
+        scput.use_shell = True
+        scput.scs = MockSplitCopyShared2()
+        remote_dir = "/var/tmp"
+        remote_file = "foobar"
+        result = scput.compare_file_sizes(100000, remote_dir, remote_file)
+        assert result == None
+
+    def test_compare_file_sizes_exec(self, monkeypatch: MonkeyPatch):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
+                stdout = "-rw-r--r--  1 foo  bar  100000 19 Dec  2019 /var/tmp/foo"
+                return True, stdout
+
+        scput = SplitCopyPut()
+        scput.scs = MockSplitCopyShared2()
+        remote_dir = "/var/tmp"
+        remote_file = "foobar"
+        result = scput.compare_file_sizes(100000, remote_dir, remote_file)
         assert result == None
 
     def test_compare_file_sizes_cmd_fail(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(self, cmd, timeout=30):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
                 return False, ""
 
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2()
-        scput.scs = MockSplitCopyShared()
+        scput.scs = MockSplitCopyShared2()
         with raises(SystemExit):
-            scput.compare_file_sizes(100000)
+            remote_dir = "/var/tmp"
+            remote_file = "foobar"
+            result = scput.compare_file_sizes(100000, remote_dir, remote_file)
 
     def test_compare_file_sizes_mismatch(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(self, cmd, timeout=30):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
                 stdout = (
                     b"foo@bar ~ % ls -l /var/tmp/foo\r\r\n"
                     b"-rw-r--r--  1 foo  bar  400000 19 Dec  2019 /var/tmp/foo\r\n"
@@ -773,14 +851,16 @@ class TestSplitCopyPut:
                 return True, stdout
 
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2()
-        scput.scs = MockSplitCopyShared()
+        scput.use_shell = True
+        scput.scs = MockSplitCopyShared2()
         with raises(SystemExit):
-            scput.compare_file_sizes(100000)
+            remote_dir = "/var/tmp"
+            remote_file = "foobar"
+            result = scput.compare_file_sizes(100000, remote_dir, remote_file)
 
     def test_remote_sha_put(self, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(self, cmd, timeout=30):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
                 stdout = (
                     "foo@bar ~ % sha224sum -a 224 /var/tmp/foo\n"
                     "d2a90f1c9edd2e9771306d8c8f4a4fc802181b973ee8167fcaff98f4 "
@@ -790,47 +870,50 @@ class TestSplitCopyPut:
                 return True, stdout
 
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2()
-        scput.scs = MockSplitCopyShared()
+        scput.scs = MockSplitCopyShared2()
         sha_bin = "sha224sum"
         sha_len = 224
         sha_hash = {224: "d2a90f1c9edd2e9771306d8c8f4a4fc802181b973ee8167fcaff98f4"}
-        result = scput.remote_sha_put(sha_bin, sha_len, sha_hash)
+        remote_dir = "/var/tmp"
+        remote_file = "foo"
+        result = scput.remote_sha_put(
+            sha_bin, sha_len, sha_hash, remote_dir, remote_file
+        )
         expected = None
         assert expected == result
 
     def test_remote_sha_put_cmd_fail(self, capsys, monkeypatch: MonkeyPatch):
-        class MockSSHShell2(MockSSHShell):
-            def run(self, cmd, timeout=30):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
                 return False, ""
 
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2()
-        scput.scs = MockSplitCopyShared()
+        scput.scs = MockSplitCopyShared2()
         sha_bin = "shasum"
         sha_len = 224
         sha_hash = {224: "d2a90f1c9edd2e9771306d8c8f4a4fc802181b973ee8167fcaff98f4"}
-        scput.remote_sha_put(sha_bin, sha_len, sha_hash)
+        remote_dir = "/var/tmp"
+        remote_file = "foo"
+        result = scput.remote_sha_put(
+            sha_bin, sha_len, sha_hash, remote_dir, remote_file
+        )
         captured = capsys.readouterr()
         assert re.search(r"remote sha hash generation failed", captured.out)
 
     def test_remote_sha_put_hash_fail(self):
-        class MockSSHShell2(MockSSHShell):
-            def run(self, cmd, timeout=30):
-                return True, ""
-
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2()
         scput.scs = MockSplitCopyShared()
         sha_bin = "shasum"
         sha_len = 224
         sha_hash = {224: "d2a90f1c9edd2e9771306d8c8f4a4fc802181b973ee8167fcaff98f4"}
         with raises(SystemExit):
-            scput.remote_sha_put(sha_bin, sha_len, sha_hash)
+            remote_dir = "/var/tmp"
+            remote_file = "foo"
+            scput.remote_sha_put(sha_bin, sha_len, sha_hash, remote_dir, remote_file)
 
     def test_remote_sha_put_hash_mismatch(self):
-        class MockSSHShell2(MockSSHShell):
-            def run(self, cmd, timeout=30):
+        class MockSplitCopyShared2(MockSplitCopyShared):
+            def ssh_cmd(*args, **kwargs):
                 stdout = (
                     "foo@bar ~ % shasum -a 224 /var/tmp/foo\n"
                     "d2a90f1c9edd2e9771306d8c8f4a4fc802181b973ee8167fceff98f4 "
@@ -840,13 +923,14 @@ class TestSplitCopyPut:
                 return True, stdout
 
         scput = SplitCopyPut()
-        scput.sshshell = MockSSHShell2()
-        scput.scs = MockSplitCopyShared()
+        scput.scs = MockSplitCopyShared2()
         sha_bin = "shasum"
         sha_len = 224
         sha_hash = {224: "d2a90f1c9edd2e9771306d8c8f4a4fc802181b973ee8167fcaff98f4"}
         with raises(SystemExit):
-            scput.remote_sha_put(sha_bin, sha_len, sha_hash)
+            remote_dir = "/var/tmp"
+            remote_file = "foo"
+            scput.remote_sha_put(sha_bin, sha_len, sha_hash, remote_dir, remote_file)
 
     def test_put_files_ftp(self):
         scput = SplitCopyPut()
